@@ -50,6 +50,8 @@
 #include "defs.h"
 #include "lib.h"
 #include "driver.h"
+#include <linux/proc_fs.h>
+#include <linux/seq_file.h>
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
@@ -109,11 +111,7 @@ static int			card_id			= 0;
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 static void scheduler (unsigned long data);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
-static irqreturn_t irq_handler (int irq, void * args, struct pt_regs * regs);
-#else
 static irqreturn_t irq_handler (int irq, void * args);
-#endif
 
 static DECLARE_TASKLET_DISABLED (scheduler_tasklet, scheduler, 0);
 
@@ -225,6 +223,7 @@ static void kill_version (card_t * card) {
 	}
 } /* kill_version */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 static void pprintf (char * page, int * len, const char * fmt, ...) {
@@ -234,9 +233,10 @@ static void pprintf (char * page, int * len, const char * fmt, ...) {
 	*len += vsprintf (page + *len, fmt, args);
 	va_end (args);
 } /* pprintf */
+#endif
 
 /*---------------------------------------------------------------------------*\
-\*-C-------------------------------------------------------------------------*/
+\*---------------------------------------------------------------------------*/
 static inline int in_critical (void) {
 	
 	return (atomic_read (&crit_level) > 0);
@@ -498,6 +498,7 @@ static char * __kcapi proc_info (struct capi_ctr * ctrl) {
 	return text;
 } /* proc_info */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 static int __kcapi ctr_info (
@@ -558,6 +559,78 @@ static int __kcapi ctr_info (
 	*start = page - ofs;
 	return ((count < len - ofs) ? count : len - ofs);
 } /* ctr_info */
+#endif
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,33)
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+static int __kcapi ctr_info (
+	struct seq_file  *mp,
+	void  *vp
+) {
+	struct capi_ctr *ctrl = mp->private;
+	card_t *		card;
+	char *			temp;
+	unsigned char		flag;
+
+	assert (ctrl != NULL);
+	card = (card_t *) ctrl->driverdata;
+	assert (card != NULL);
+	seq_printf(mp, "%-16s %s\n", "name", SHORT_LOGO);
+	seq_printf(mp, "%-16s 0x%04x\n", "io", card->base);
+	seq_printf(mp, "%-16s %d\n", "irq", card->irq);
+	temp = card->version ? card->string[1] : "A1";
+	seq_printf(mp, "%-16s %s\n", "type", temp);
+	temp = card->version ? card->string[0] : "-";
+#if defined (__fcclassic__) || defined (__fcpcmcia__)
+	seq_printf(mp, "%-16s 0x%04x\n", "revision", card->info);
+#elif defined (__fcpci__)
+	seq_printf(mp, "%-16s %d\n", "class", card_id);
+#endif
+	seq_printf(mp, "%-16s %s\n", "ver_driver", temp);
+	seq_printf(mp, "%-16s %s\n", "ver_cardtype", SHORT_LOGO);
+
+	flag = ((unsigned char *) (ctrl->profile.manu))[3];
+	if (flag) {
+		seq_printf(mp, "%-16s%s%s%s%s%s%s%s\n", "protocol",
+			(flag & 0x01) ? " DSS1" : "",
+			(flag & 0x02) ? " CT1" : "",
+			(flag & 0x04) ? " VN3" : "",
+			(flag & 0x08) ? " NI1" : "",
+			(flag & 0x10) ? " AUSTEL" : "",
+			(flag & 0x20) ? " ESS" : "",
+			(flag & 0x40) ? " 1TR6" : ""
+		);
+	}
+	flag = ((unsigned char *) (ctrl->profile.manu))[5];
+	if (flag) {
+		seq_printf(mp, "%-16s%s%s%s%s\n", "linetype",
+			(flag & 0x01) ? " point to point" : "",
+			(flag & 0x02) ? " point to multipoint" : "",
+			(flag & 0x08) ? " leased line without D-channel" : "",
+			(flag & 0x04) ? " leased line with D-channel" : ""
+		);
+	}
+	return 0;
+} /* ctr_info */
+#endif
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,33)
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+static int ctr_proc_open(struct inode *inode, struct file *file)
+{
+       return single_open(file, ctr_info, PDE(inode)->data);
+}
+
+const struct file_operations ctr_proc_fops = {
+       .owner          = THIS_MODULE,
+       .open           = ctr_proc_open,
+       .read           = seq_read,
+       .llseek         = seq_lseek,
+       .release        = single_release,
+};
+#endif
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
@@ -640,7 +713,11 @@ int __kcapi add_card (struct capi_driver * drv, capicardparams * args) {
 	ctrl->release_appl =	release_appl;
 	ctrl->send_message =	send_msg;
 	ctrl->procinfo =	proc_info;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 	ctrl->ctr_read_proc =	ctr_info;
+#else
+	ctrl->proc_fops =	&ctr_proc_fops;
+#endif
 	if (0 != (res = attach_capi_ctr (ctrl))) {
 		dec_use_count ();
 		stop (card);
@@ -850,17 +927,10 @@ static void scheduler (unsigned long data) {
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
-static irqreturn_t irq_handler (int irq, void * args, struct pt_regs * regs) {
-#else
 static irqreturn_t irq_handler (int irq, void * args) {
-#endif
 	int	res	= IRQ_NONE;
 	
 	UNUSED_ARG (irq);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
-	UNUSED_ARG (regs);
-#endif
 	if (args != NULL) {
 		assert (capi_lib->cm_handle_events != NULL);
 		if (atomic_read (&scheduler_id) == smp_processor_id ()) {
